//////////////////////////////////////////////////////////////////////////////
// File      : qd02demo.c
// Function  : Demo application for Orled QD02 ringlight controller
// Copyright : (C) 2015 Orled
// ----------------------------------------------
// Build dependencies   : OrledQD02.lib
// Runtime dependencies : OrledQD02.dll
//////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <wchar.h>
#include <string.h>
#include <stdlib.h>
#include <conio.h>
#include <time.h>
#include <windows.h>

#include "..\c_include\qd02_api.h"               // Ring light API

#define LOGICAL_ADDRESS     0       // Connect to the QD02 that has this device address (assigned via rotary switch setting on QD02)

// Change font color.
static HANDLE hConsole;  // Handle to user console
#define UserFont()      SetConsoleTextAttribute(hConsole, FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_RED)          // Echo/prompt:     white
#define NotifyFont()    SetConsoleTextAttribute(hConsole, FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_INTENSITY)    // System state:    cyan
#define FaultFont()     SetConsoleTextAttribute(hConsole, FOREGROUND_RED | FOREGROUND_INTENSITY)                        // System fault:    red
#define WarningFont()   SetConsoleTextAttribute(hConsole, FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_INTENSITY)     // Warning/notice:  yellow

//////////////////////////////////////////////////
// Console prompt.

static void Prompt(void)
{
    UserFont();
    printf("\n>");
}

///////////////////////////////////////////////////////////////////////
// Error handler.

static int errproc(int errcode)
{
    if ((errcode != QD02_ERR_OK) && (errcode != QD02_ERR_TIMEOUT))  // If fatal error ...
    {
        QD02_Close(LOGICAL_ADDRESS);            // Close device

        FaultFont();                            // Show error message
        switch (errcode)
        {
        case QD02_ERR_DEVICE_NOT_FOUND: printf("\nCan't find QD02 at specified logical address\n"); break;
        case QD02_ERR_DEVICE_OPEN:      printf("\nCan't open QD02\n"); break;
        case QD02_ERR_DEVICE_CLOSED:    printf("\nQD02 is not open\n"); break;
        case QD02_ERR_READ:             printf("\nDevice read error\n"); break;
        case QD02_ERR_WRITE:            printf("\nDevice write error\n"); break;
        case QD02_ERR_INVALID_ARG:      printf("\nIllegal address or argument value\n"); break;
        case QD02_ERR_OPENHIDLIB:       printf("\nProblem opening or linking to hid.dll\n"); break;
        case QD02_ERR_COMMAND_MISMATCH: printf("\nCommand/reply mismatch\n"); break;
        case QD02_ERR_CREATE_RESOURCE:  printf("\nCan't create resource\n"); break;
        case QD02_ERR_WAS_SHUTDOWN:     printf("\nReceive comm terminated\n"); break;
        default :                       printf("\nUnknown error: %d\n", errcode); break;
        }
        WarningFont();
        printf("Hit any key to exit ... ");     // Wait for user ack
        do {} while (!_kbhit());
        _getch();
        exit(0);                                // Terminate app
    }
    return errcode;  // non-fatal error
}

//////////////////////////////////////////////////////////////
// Display a notification message received from the QD02.

static DisplayNotification(QD02_NOTIFICATION *n)
{
    NotifyFont(); // switch to notification color

    // Uncomment these if desired:
//    printf("%6d:", n->SystemTime);
//    printf("%6d: ", n->NotificationID);

    if (n->EventFlags & NOTIFY_SOFT)    // if host called QD02_TriggerNotification() then display all
        n->EventFlags |= (NOTIFY_POWER | NOTIFY_CONSOLE | NOTIFY_FAULT | NOTIFY_CONFIG | NOTIFY_TEMP);
    if ((n->StatusFlags & (STATUS_CABLEFAULT | STATUS_KEYPADFAULT | STATUS_OVERTEMP)) != 0)
        n->EventFlags |= NOTIFY_FAULT;  // always display faults

    if (n->EventFlags & NOTIFY_STROBE) { // flash strobe was activated
        printf("STROBE ");
    }

    if (n->EventFlags & NOTIFY_POWER) { // power state change
        printf("%s ", (n->StatusFlags & STATUS_POWERSWITCH) ? "ON " : "OFF");
    }

    if (n->EventFlags & NOTIFY_CONSOLE) { // console state change
        printf("q=%d%d%d%d i=%d x=%d ",
            (n->QuadrantEnables & 8) != 0,
            (n->QuadrantEnables & 4) != 0,
            (n->QuadrantEnables & 2) != 0,
            (n->QuadrantEnables & 1) != 0,
            n->QuadrantIntensity,
            n->CrosshairEnable
            );
    }

    if (n->EventFlags & NOTIFY_FAULT) { // fault or status change
        if ((n->StatusFlags & STATUS_LASERDETECTED) == 0) {
            WarningFont();  // use warning color
            printf("%s", "NOX ");
            NotifyFont();   // resume notification color
        }
        if ((n->StatusFlags & (STATUS_CABLEFAULT | STATUS_KEYPADFAULT | STATUS_OVERTEMP)) == 0)
            printf("OK ");  // no faults
        else {
            FaultFont();    // use fault color
            printf("%s%s%s",
                (n->StatusFlags & STATUS_OVERTEMP)    ? "TMP " : "",
                (n->StatusFlags & STATUS_CABLEFAULT)  ? "CBL " : "",
                (n->StatusFlags & STATUS_KEYPADFAULT) ? "KBD " : ""
                );
            NotifyFont();   // resume notification color
        }
    }

    if (n->EventFlags & NOTIFY_CONFIG) { // configuration change
        char *IlluminationMode[] = {"PWM", "ANA", "STB"};
        printf("%s xt=%d sd=%d st=%d imax=%d ",
            (n->IlluminationMode < 3) ? IlluminationMode[n->IlluminationMode] : "???",
            n->CrosshairOntime, n->StrobeDelay, n->StrobeDuration, n->MaxIntensity);
    }

    if (n->EventFlags & NOTIFY_TEMP) { // quadrant temperature change
        int i;
        printf("t=");
        for (i = 0; i < 4; i++)
            printf("%d ", n->Temperature[i]);
    }

    Prompt();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// To keep the demo simple, polling is used to process received QD02 notification messages. A more efficient
// (and higher performance) method would be to employ a separate thread to handle notifications. The thread
// would wait for each notification (by calling QD02_ReadNotification() with a non-zero wait_msec time), and
// then process the notification and loop to wait for the next one.

static void Poll(int delay)
{
    clock_t tstart = clock();
    do
    {
        QD02_NOTIFICATION n;
        if (errproc(QD02_ReadNotification(LOGICAL_ADDRESS, &n, 0)) == QD02_ERR_OK)      // If a notification message was received from the QD02, fetch it and
            DisplayNotification(&n);                                                    //   display it
    }
    while (((double)(clock() - tstart) / CLOCKS_PER_SEC * 1000) < delay);
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////  AUTOMATION DEMO  ///////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Helper functions
static void LaserToggle(int stepdelay)  // toggle laser crosshairs on/off
{
    errproc(QD02_ControlCrosshair(LOGICAL_ADDRESS, 1));  // on
    Poll(stepdelay);
    errproc(QD02_ControlCrosshair(LOGICAL_ADDRESS, 0));  // off
    Poll(stepdelay);
}

static void RotateQuadrants(int numsteps, int stepdelay)    // "spin" the ringlights
{
    int q;
    for (q = 0; q < numsteps; q++) {
        errproc(QD02_ControlQuadrants(LOGICAL_ADDRESS, 1 << (q & 3)));
        Poll(stepdelay);
    }
}

static void BrightnessUpDown(int stepdelay)     // ramp brightness up/down
{
    static const unsigned char intensity[] = {1,2,3,4,5,6,7,8,7,6,5,4,3,2,1};
    int step;
    for (step = 0; step < sizeof(intensity); step++) {
        errproc(QD02_ControlIntensity(LOGICAL_ADDRESS, intensity[step]));
        Poll(stepdelay);
    }
}

// The automation sequencer.
static void DemoSequencer(void)
{
    int q;
    Prompt();
    WarningFont();
    printf("Starting automation demo"); 
    Prompt();
    errproc(QD02_ControlPower(LOGICAL_ADDRESS, 1));                             // turn power on
    errproc(QD02_ControlQuadrants(LOGICAL_ADDRESS, 15));                        // enable all quadrants
    errproc(QD02_ControlIntensity(LOGICAL_ADDRESS, 1)); Poll(500);              // switch to intensity level 1 (lowest)
    for (q = 4; q > 0; q--)                                                     // toggle crosshairs on/off a few times
        LaserToggle(200 * q);

    for (q = 0; q < 62; q++) {                                                  // "spin" the light while changing intensity
        int rotations = 10;
        errproc(QD02_ControlQuadrants(LOGICAL_ADDRESS, 1 << (q & 3)));
        Poll(rotations + 2 * (61 - q));
        if ((q & 3) == 3)
            QD02_ControlIntensity(LOGICAL_ADDRESS, (q + 2) >> 2);
    }
    RotateQuadrants(80, 10);                                                    // "spin" the light faster and faster
    RotateQuadrants(120, 5);
    RotateQuadrants(400, 1);
    for (q = 0; q < 8; q++) {                                                   // ramp brightness up/down with only one quadrant enabled at a time
        QD02_ControlQuadrants(LOGICAL_ADDRESS, 1 << (q & 3));
        BrightnessUpDown(25);
    }
    errproc(QD02_ControlQuadrants(LOGICAL_ADDRESS, 15));                        // enable all quadrants
    BrightnessUpDown(25);                                                       // ramp brightness up/down quickly
    BrightnessUpDown(100);                                                      // ramp brightness up/down slowly
    errproc(QD02_ConfigMode(LOGICAL_ADDRESS, ILLUMINATION_MODE_STROBE));        // switch to strobe mode
    errproc(QD02_TriggerStrobe(LOGICAL_ADDRESS)); Poll(1000);                   // trigger some strobes
    errproc(QD02_TriggerStrobe(LOGICAL_ADDRESS)); Poll(800);
    errproc(QD02_TriggerStrobe(LOGICAL_ADDRESS)); Poll(500);
    errproc(QD02_ConfigMode(LOGICAL_ADDRESS, ILLUMINATION_MODE_PWM));           // switch to constant color mode
    errproc(QD02_ControlPower(LOGICAL_ADDRESS, 0)); Poll(10);                   // turn power off and poll briefly to process all remaining notifications
    WarningFont();
    printf("End automation demo");
    Prompt();
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////  MAIN FUNCTION  ////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char* argv[])
{
    static const char HwRev[] = "0ABCDEFGHJKLMNPRSTUVWXYZ";
    unsigned short      hw_version;
    QD02_FIRMWAREINFO   fw;
    char                c = '?';    // display menu at startup

    hConsole = GetStdHandle(STD_OUTPUT_HANDLE);                     // get handle to console

    if (QD02_Open(LOGICAL_ADDRESS, &hw_version) != QD02_ERR_OK)     // open ringlight and get hardware version number
    {
        printf("unable to open device\n");
        do {} while (!_kbhit());
        _getch();
        return 1;
    }

    errproc(QD02_ReadFirmwareInfo(LOGICAL_ADDRESS, &fw));           // get firmware version
    printf("Found device: rev %c, firmware %d.%d.%d\n",
        HwRev[hw_version],
        fw.FwVersion >> 24, (fw.FwVersion >> 16) & 0xFF, (fw.FwVersion & 0xFFFF));

    errproc(QD02_TriggerNotification(LOGICAL_ADDRESS));             // force QD02 to send notification message; this will display state info after initial menu

    do  // Repeat until app closed -------------------------
    {
        switch (c)
        {
        case '?':
        case 'M':
            printf("\nCOMMAND MENU\n");         // Display menu
            printf(" X - demo sequencer\n");
            printf(" N - power on\n");
            printf(" F - power off\n");
            printf(" A - analog mode\n");
            printf(" B - digital mode\n");
            printf(" Z - strobe mode\n");
            printf(" Y - config crosshair\n");
            printf(" 1 - crosshair on\n");
            printf(" 0 - crosshair off\n");
            printf(" S - config strobe\n");
            printf(" T - trigger strobe\n");
            printf(" J - enable temp notifications\n");
            printf(" K - disable temp notifications\n");
            printf(" W - save config\n");
            printf(" R - restore config\n");
            printf(" Q - quit");
            break;

        case 'X': DemoSequencer();                                                          break;  // Automation demo
            // Power
        case 'N': errproc(QD02_ControlPower(LOGICAL_ADDRESS, 1));                           break;  // On
        case 'F': errproc(QD02_ControlPower(LOGICAL_ADDRESS, 0));                           break;  // Off
            // Illumination mode
        case 'A': errproc(QD02_ConfigMode(LOGICAL_ADDRESS, ILLUMINATION_MODE_ANALOG));      break;  // Analog mode
        case 'B': errproc(QD02_ConfigMode(LOGICAL_ADDRESS, ILLUMINATION_MODE_PWM));         break;  // Digital mode
        case 'Z': errproc(QD02_ConfigMode(LOGICAL_ADDRESS, ILLUMINATION_MODE_STROBE));      break;  // Strobe mode
            // Crosshair
        case 'Y': errproc(QD02_ConfigCrosshair(LOGICAL_ADDRESS, 5000));                     break;  // Config crosshair on-time to 5 seconds
        case '1': errproc(QD02_ControlCrosshair(LOGICAL_ADDRESS, 1));                       break;  // crosshair on
        case '0': errproc(QD02_ControlCrosshair(LOGICAL_ADDRESS, 0));                       break;  // crosshair off
            // Strobe
        case 'S': errproc(QD02_ConfigStrobe(LOGICAL_ADDRESS, 17, 10000));                   break;  // Config strobe for 17us trigger delay, 10ms duration
        case 'T': errproc(QD02_TriggerStrobe(LOGICAL_ADDRESS));                             break;  // trigger strobe
            // System status/control
        case 'J': errproc(QD02_MonitorTemp(LOGICAL_ADDRESS, 1));                            break;  // Enable notification upon temperature change
        case 'K': errproc(QD02_MonitorTemp(LOGICAL_ADDRESS, 0));                            break;  // Disable notification upon temperature change
            // Defaults read/write
        case 'W': errproc(QD02_ConfigSave(LOGICAL_ADDRESS));                                break;  // Save config as new defaults
        case 'R': errproc(QD02_ConfigRestore(LOGICAL_ADDRESS));                             break;  // Restore config to defaults
        }

        if (c != 'X')               // Prompt if not completing automation demo
            Prompt();

        while (!_kbhit())           // Poll notifications while waiting for user command
            Poll(0);
        c = toupper(_getch());

        printf("%c", c);
    }
    while (c != 'Q');

    QD02_Close(LOGICAL_ADDRESS);  // close ringlight
    return 0;
}


