////////////////////////////////////////////////////////////////////////////
// File      : qd02_api.h
// Function  : Include for QD02 application programs
// Copyright : (C) 2015 Orled
////////////////////////////////////////////////////////////////////////////

#ifndef _QD02_API_H_
#define _QD02_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _BUILD_QD02_MIDDLEWARE_
#ifdef _WIN32
#define QD02_API
#define QD02_CC     __stdcall
#endif
#endif

// Manifest constants -----------------------------------

#define QD02_MAX_DEVICES            8           // Maximum number of QD02 controllers supported in one system

// Error codes ------------------------------------------

// Normal operation
#define QD02_ERR_OK                 0           // No error
#define QD02_ERR_DEVICE_NOT_FOUND   (-1)        // Can't find QD02 at specified logical address
#define QD02_ERR_DEVICE_OPEN        (-2)        // Can't open QD02
#define QD02_ERR_DEVICE_CLOSED      (-3)        // QD02 is not open
#define QD02_ERR_READ               (-4)        // Error during device read
#define QD02_ERR_WRITE              (-5)        // Error during device write
#define QD02_ERR_INVALID_ARG        (-6)        // Illegal logical address or other function argument
#define QD02_ERR_OPENHIDLIB         (-7)        // Problem opening or can't find the generic hid api (in Windows, this is hid.dll)
#define QD02_ERR_COMMAND_MISMATCH   (-8)        // Command/reply mismatch (can result from multiple threads/processes attempting to talk to same device)
#define QD02_ERR_TIMEOUT            (-9)        // Timed out while waiting for device I/O
#define QD02_ERR_CREATE_RESOURCE    (-10)       // Can't create resource (e.g., mutex/semaphore/thread)
#define QD02_ERR_WAS_SHUTDOWN       (-11)       // Receive communications have terminated
// factory use only:
#define QD02_ERR_UPLOAD_FIRMWARE    (-100)      // Failed to upload firmware; see fwerr for details
#define QD02_ERR_KB_NAK             (-200)      // Keypad command caused I2C NAK
#define QD02_ERR_KB_ACTIVE          (-201)      // Keypad command rejected because scanning is active
#define QD02_ERR_KB_BUSY            (-202)      // Keypad command rejected because previous command not completed

// Illumination modes ------------------------------------

#define ILLUMINATION_MODE_PWM       0           // PWM intensity control (constant color mode) -- factory default
#define ILLUMINATION_MODE_ANALOG    1           // Analog intensity control (constant light mode)
#define ILLUMINATION_MODE_STROBE    2           // Strobe mode

// Status flags ------------------------------------------

#define STATUS_POWERSWITCH          0x80        // Power switch is on
#define STATUS_LASERDETECTED        0x40        // Laser crosshair feature detected
#define STATUS_OVERTEMP             0x10        // Excessive high temperature on one or more ringlight quadrants; intensity is being automatically limited
#define STATUS_CABLEFAULT           0x02        // Disconnected ringlight cable (or ringlight power supply problem)
#define STATUS_KEYPADFAULT          0x01        // Internal keypad fault

// Notification event flags ------------------------------

#define NOTIFY_SOFT                 0x80        // Software triggered notification
#define NOTIFY_SPARE                0x40        //   reserved for future use
#define NOTIFY_STROBE               0x20        // Flash strobe fired
#define NOTIFY_POWER                0x10        // Power switch state change
#define NOTIFY_CONFIG               0x08        // Configuration change (illumination mode, strobe attributes)
#define NOTIFY_CONSOLE              0x04        // Console change (crosshair state, quadrant enables, intensity)
#define NOTIFY_FAULT                0x02        // Fault or other status change (keypad, cable, firmware upgrade, laser detect)
#define NOTIFY_TEMP                 0x01        // Periodically invoked, used for automatic temperature updates

// Structure types ---------------------------------------

typedef struct QD02_NOTIFICATION {              // NOTIFICATION MESSAGE FROM RING LIGHT CONTROLLER -----------------------
                                                // Notification attributes
    unsigned char   EventFlags;                 //   Notification event flags; these indicate what triggered the notification message.
    unsigned short  NotificationID;             //   Unique identifier for notification message. This is incremented for each transmitted message.
    unsigned short  SystemTime;                 //   Timestamp indicating when notification was sent. Resolution = 1 millisecond; overflows every 65.536 seconds.
                                                // Console state and controller status
    unsigned char   QuadrantEnables;            //   Enable flags for ring light quadrants: (x, x, x, x, Q3, Q2, Q1, Q0).
    unsigned char   QuadrantIntensity;          //   Ringlight intensity level in range [1:8].
    unsigned char   CrosshairEnable;            //   Boolean: Enable crosshair laser illumination.
    unsigned char   StatusFlags;                //   Status flags.
                                                // Quadrant temperatures
    unsigned char   Temperature[4];             //   Temperature, in degrees C, of the four ring light quadrants.
                                                // Configuration settings
    unsigned char   IlluminationMode;           //   Illumination control mode: ILLUMINATION_MODE_PWM | ILLUMINATION_MODE_ANALOG | ILLUMINATION_MODE_STROBE.
    unsigned short  CrosshairOntime;            //   Crosshairs will automatically turn off after this many microseconds.
    unsigned short  StrobeDelay;                //   Strobe trigger delay in microseconds.
    unsigned short  StrobeDuration;             //   Strobe width in 4*microseconds units.
    unsigned char   MaxIntensity;               //   Brightness of max intensity setting, expressed as percentage of max possible brightness.
                                                // Additional info
    unsigned int    DroppedNotifications;       //   Number of dropped notifications since previous call to QD02_ReadNotification() -- generated by API.
} QD02_NOTIFICATION;

typedef struct QD02_FIRMWAREINFO {
    unsigned char   isBootloader;               // Boolean: running boot loader firmware
    unsigned int    FwVersion;                  // QD02 firmware version number
} QD02_FIRMWAREINFO;

typedef struct QD02_TASKINFO {  // reserved for factory use only
    char            name[4];
    unsigned short  stackmargin;
    unsigned short  exectime;
} QD02_TASKINFO;

// API functions --------------------------------------------------------

QD02_API int QD02_CC QD02_Open(int logaddr, unsigned short *hw_version);
QD02_API int QD02_CC QD02_Close(int logaddr);
QD02_API int QD02_CC QD02_Shutdown(int logaddr);
QD02_API int QD02_CC QD02_Enumerate(void);
QD02_API int QD02_CC QD02_ConfigStrobe(int logaddr, unsigned short delay, unsigned int duration);
QD02_API int QD02_CC QD02_ConfigMode(int logaddr, unsigned char mode);
QD02_API int QD02_CC QD02_ConfigMaxIntensity(int logaddr, unsigned char intensity);
QD02_API int QD02_CC QD02_ConfigCrosshair(int logaddr, unsigned short ontime);
QD02_API int QD02_CC QD02_ConfigSave(int logaddr);
QD02_API int QD02_CC QD02_ConfigRestore(int logaddr);
QD02_API int QD02_CC QD02_ControlPower(int logaddr, unsigned char enable);
QD02_API int QD02_CC QD02_ControlQuadrants(int logaddr, unsigned char enables);
QD02_API int QD02_CC QD02_ControlIntensity(int logaddr, unsigned char level);
QD02_API int QD02_CC QD02_ControlCrosshair(int logaddr, unsigned char enable);
QD02_API int QD02_CC QD02_MonitorTemp(int logaddr, unsigned char enable);
QD02_API int QD02_CC QD02_TriggerStrobe(int logaddr);
QD02_API int QD02_CC QD02_TriggerNotification(int logaddr);
QD02_API int QD02_CC QD02_ReadNotification(int logaddr, QD02_NOTIFICATION *notification, int wait_msec);
QD02_API int QD02_CC QD02_ReadStatistics(int logaddr, unsigned char taskid, QD02_TASKINFO *info);
QD02_API int QD02_CC QD02_ReadFirmwareInfo(int logaddr, QD02_FIRMWAREINFO *info);
QD02_API int QD02_CC QD02_ApiVersionRead(void);

// Reserved for factory use only:
QD02_API int QD02_CC QD02_UploadFirmware(char *hexbuf, int buflen, int *fwerr);
QD02_API int QD02_CC QD02_KeypadProgram(int logaddr);
QD02_API int QD02_CC QD02_KeypadErase(int logaddr);
QD02_API int QD02_CC QD02_Reboot(int logaddr);


#ifdef __cplusplus
}
#endif


#endif  // ifndef _QD02_API_H_


