﻿Imports System.Threading

Public Class frmQD02

    '*******************************************************************************************
    ' File      : frmQD02.vb
    ' Function  : Demo app for Orled's QD02 ringlight controller
    ' Copyright : (C) 2015 Orled, Inc.
    '*******************************************************************************************

    Private Const SAVE = 1                      ' 1 = save changes to flash, 0 = don't save changes to flash
    Private Const AUTOUPDATE_INTERVAL = 250     ' Update temperatures once per AUTOUPDATE_INTERVAL milliseconds
    Private WithEvents qd02 As QD02Device       ' QD02 interface object
    Private DetectedFatalError As Boolean = False

    Private state As QD02Device.NOTIFICATION = New QD02Device.NOTIFICATION  ' System state. This is maintained by the monitor thread. KeypadClick() uses it to toggle button states.

    '************************************************************************************************
    ' FORM STARTUP/SHUTDOWN
    '************************************************************************************************

    Public Sub New(ByVal device_logaddr As Integer)
        MyBase.New()
        InitializeComponent()
        qd02 = New QD02Device(device_logaddr)   ' Instantiate the QD02 interface
        Me.Text = "QD02 Demo (" & device_logaddr.ToString & ")"
    End Sub

    Private Sub Form_Load(ByVal s As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.Show()                                           ' Init form so we can access its controls
        qd02.Open(Me)                                       ' Open QD02 device
        lblApiVersion.Text = GetApiVersionString()          ' Display API version number
        lblHardwareVersion.Text = "Rev " & qd02.HardwareRev ' Display hardware version
        lblFirmwareVersion.Text = qd02.FirmwareVersion      ' Display firmware version
        qd02.MonitorTemp(True)                              ' Enable automatic temperature updates
    End Sub

    Private Sub Form_Closing(ByVal sender As Object, ByVal e As FormClosingEventArgs) Handles MyBase.Closing
        If Me.Enabled Or DetectedFatalError Then qd02.Close() Else e.Cancel = True ' Abort if automation demo is running, otherwise close the QD02
    End Sub

    '************************************************************************************************
    ' NOTIFICATION MONITOR -- updates form controls to show current ring light state
    '************************************************************************************************

    Private Sub DisplayNotification(ByVal notification As QD02Device.NOTIFICATION) Handles qd02.ReceivedNotification
        Static ActiveGood As System.Drawing.Color
        Static ActiveBad As System.Drawing.Color = Color.Red
        state = notification    ' Publish notification to public view
        With state
            If (.EventFlags And (QD02Device.EVENT_FLAGS.SOFT Or QD02Device.EVENT_FLAGS.POWER)) Then ' update virtual power button
                ActiveGood = IIf(.StatusFlags And QD02Device.STATUS.POWER, Color.Lime, Color.DarkGray)
                btnPower.BackColor = IIf(.StatusFlags And QD02Device.STATUS.POWER, ActiveGood, Color.LightGray)
                barIntensity.ForeColor = ActiveGood
            End If
            If (.EventFlags And (QD02Device.EVENT_FLAGS.SOFT Or QD02Device.EVENT_FLAGS.POWER Or QD02Device.EVENT_FLAGS.CONSOLE)) Then ' display virtual keypad
                btnCrosshair.BackColor = IIf(.CrosshairEnable, ActiveGood, SystemColors.Control)
                btnQuadrant1.BackColor = IIf(.QuadrantEnables And 1, ActiveGood, SystemColors.Control)
                btnQuadrant2.BackColor = IIf(.QuadrantEnables And 2, ActiveGood, SystemColors.Control)
                btnQuadrant3.BackColor = IIf(.QuadrantEnables And 4, ActiveGood, SystemColors.Control)
                btnQuadrant4.BackColor = IIf(.QuadrantEnables And 8, ActiveGood, SystemColors.Control)
                barIntensity.Value = .Brightness
            End If
            If (.EventFlags And (QD02Device.EVENT_FLAGS.SOFT Or QD02Device.EVENT_FLAGS.POWER Or QD02Device.EVENT_FLAGS.FAULT)) Then ' display fault status
                lblKeypadFault.BackColor = IIf(.StatusFlags And QD02Device.STATUS.KEYPADFAULT, ActiveBad, SystemColors.Control)
                lblCableFault.BackColor = IIf(.StatusFlags And QD02Device.STATUS.CABLEFAULT, ActiveBad, SystemColors.Control)
                lblOvertemp.BackColor = IIf(.StatusFlags And QD02Device.STATUS.OVERTEMP, ActiveBad, SystemColors.Control)
                btnCrosshair.Visible = ((.StatusFlags And QD02Device.STATUS.LASERHEAD) <> 0) ' crosshair button visible only for compatible ring light heads
            End If
            If (.EventFlags And (QD02Device.EVENT_FLAGS.SOFT Or QD02Device.EVENT_FLAGS.TEMP)) Then ' display quadrant temperatures
                lblTemperature0.Text = .Temperature(0).ToString
                lblTemperature1.Text = .Temperature(1).ToString
                lblTemperature2.Text = .Temperature(2).ToString
                lblTemperature3.Text = .Temperature(3).ToString
            End If
            If (.EventFlags And (QD02Device.EVENT_FLAGS.SOFT Or QD02Device.EVENT_FLAGS.CONFIG)) Then  ' display illumination mode and configuration settings
                Static rbMode() As RadioButton = {rbModePwm, rbModeAnalog, rbModeStrobe}
                rbMode(.IlluminationMode).Checked = True
                btnStrobe.Enabled = (.IlluminationMode = QD02Device.ILLUMINATION_MODE.STROBE)
                txtCrosshairOntime.Text = .CrosshairOntime.ToString
                txtStrobeDelay.Text = .StrobeDelay.ToString
                txtStrobeDuration.Text = (CUInt(.StrobeDuration) << 2).ToString     ' 4us/bit
                txtMaxIntensity.Text = .MaxIntensity.ToString
            End If
            If (.EventFlags And QD02Device.EVENT_FLAGS.STROBE) Then
                btnStrobe.BackColor = ActiveGood
                StrobeTimer.Enabled = True
            End If
        End With
    End Sub

    '************************************************************************************************
    ' AUTOMATION DEMO THREAD
    '************************************************************************************************

    ' DEMO button handler.
    Private Sub btnDemo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDemo.Click
        GUIEnable(False)     ' disable all UI controls
        Dim DemoThread As New Threading.Thread(AddressOf AutomationSequencerThread)                         ' Automation demo thread.
        DemoThread.Start()
    End Sub

    ' Demo helper functions
    Private Sub RampIntensityUpDown(ByVal delay As Integer)    ' Ramp intensity up/down.
        For i As Integer = 0 To 8 : qd02.Intensity = i : Thread.Sleep(delay) : Next
        For i = 8 To 0 Step -1 : qd02.Intensity = i : Thread.Sleep(delay) : Next
    End Sub

    Private Sub RotateQuadrants(ByVal delay As Integer)    ' Rotate quadrant enables.
        For quadrant As Integer = 0 To 3 : qd02.Quadrants = 1 << (quadrant And 3) : Thread.Sleep(delay) : Next
    End Sub

    Private Sub LaserCycle(ByVal delay As Integer)    ' Toggle lasers.
        qd02.Crosshair = True : Thread.Sleep(delay)
        qd02.Crosshair = False : Thread.Sleep(delay)
    End Sub

    Private Sub AutomationSequencerThread()  ' DEMO AUTOMATION THREAD ===============================

        Dim rotate As Integer = 20        ' Configuration constants in milliseconds
        Dim slew As Integer = 40

        Dim originalState As QD02Device.NOTIFICATION = state                            ' Cache starting state
        qd02.IlluminationMode = QD02Device.ILLUMINATION_MODE.DIGITAL                    ' Switch to digital intensity control mode, don't save config
        qd02.Intensity = 1                                                              ' Set to minimum ringlight intensity (level 1)
        qd02.Quadrants = 0                                                              ' Turn off all quadrants
        qd02.Power = True                                                               ' Turn on power
        Thread.Sleep(200)
        For seq = 500 To 100 Step -100 : LaserCycle(seq) : Next : Thread.Sleep(200)     ' Cycle laser crosshairs
        qd02.Quadrants = 15 : Thread.Sleep(500)                                         ' Turn on all quadrants
        For seq = 500 To 100 Step -100 : LaserCycle(seq) : Next : Thread.Sleep(200)     ' Cycle laser crosshairs
        For seq As Integer = 150 To 10 Step -10 : RotateQuadrants(seq) : Next           ' Rotate quadrants
        For seq = 1 To 20 : RotateQuadrants(1) : Next
        For seq = 1 To 2                                                                ' Ramp intensity up/down for each quadrant in turn
            For quadrant As Integer = 0 To 3 : qd02.Quadrants = 1 << (quadrant And 3) : RampIntensityUpDown(25 / seq) : Next
        Next
        qd02.Quadrants = 15 : RampIntensityUpDown(slew * 2)                             ' Ramp intensity of all quadrants up/down
        qd02.IlluminationMode = QD02Device.ILLUMINATION_MODE.STROBE : Thread.Sleep(500) ' Switch to strobe mode and output some strobes
        qd02.ConfigStrobe(0, 15000)
        For seq = 900 To 100 Step -200 : qd02.TriggerStrobe() : Thread.Sleep(seq) : Next
        qd02.TriggerStrobe() : Thread.Sleep(100)
        qd02.IlluminationMode = originalState.IlluminationMode                          ' Restore original state
        qd02.Intensity = originalState.Brightness
        qd02.Quadrants = originalState.QuadrantEnables
        qd02.Power = ((originalState.StatusFlags And QD02Device.STATUS.POWER) <> 0)
        qd02.ConfigStrobe(originalState.StrobeDelay, originalState.StrobeDuration)
        GUIEnable(True)         ' enable all UI controls
    End Sub

    '************************************************************************************************
    ' GUI BUTTON EVENT HANDLERS
    '************************************************************************************************

    Private Sub LoseFocus()
        lblDummyFocus.Focus()   ' take focus off clicked button
    End Sub

    ' Handler for illumination mode selectors
    Private Sub IlluminationModeChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rbModePwm.Click, rbModeAnalog.Click, rbModeStrobe.Click
        Select Case CType(sender, RadioButton).Text
            Case "Digital" : qd02.IlluminationMode = QD02Device.ILLUMINATION_MODE.DIGITAL     ' Change mode
            Case "Analog" : qd02.IlluminationMode = QD02Device.ILLUMINATION_MODE.ANALOG
            Case "Strobe" : qd02.IlluminationMode = QD02Device.ILLUMINATION_MODE.STROBE
        End Select
        qd02.ConfigSave()    ' save mode as new default
    End Sub

    ' Handler for virtual keypad buttons
    Private Sub VirtualKeypadClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _
     btnQuadrant1.Click, btnQuadrant2.Click, btnQuadrant3.Click, btnQuadrant4.Click, btnIntensityIncrease.Click, btnIntensityDecrease.Click, btnPower.Click, btnCrosshair.Click
        Select Case CType(sender, Button).Tag
            Case "1" : qd02.Quadrants = state.QuadrantEnables Xor 1
            Case "2" : qd02.Quadrants = state.QuadrantEnables Xor 2
            Case "3" : qd02.Quadrants = state.QuadrantEnables Xor 4
            Case "4" : qd02.Quadrants = state.QuadrantEnables Xor 8
            Case "+" : qd02.Intensity = state.Brightness + 1
            Case "-" : qd02.Intensity = state.Brightness - 1
            Case "X" : qd02.Crosshair = state.CrosshairEnable Xor 1
            Case "P" : qd02.Power = (state.StatusFlags And QD02Device.STATUS.POWER) Xor QD02Device.STATUS.POWER
        End Select
        LoseFocus()   ' take focus off clicked button
    End Sub

    ' Handler for config button
    Private Sub ConfigClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnConfig.Click
        qd02.CrosshairOntime = TextboxToInt(txtCrosshairOntime, 1, 65535)
        qd02.ConfigStrobe(CUShort(TextboxToInt(txtStrobeDelay, 0, 65535)), CUShort(TextboxToInt(txtStrobeDuration, 4, 262140) >> 2))  ' strobe duration = 4us/bit
        qd02.MaxIntensity = TextboxToInt(txtMaxIntensity, 0, 200)
        qd02.ConfigSave()   ' save current config as defaults
        LoseFocus()         ' take focus off clicked button
    End Sub

    ' Handler for strobe trigger button
    Private Sub StrobeTriggerClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStrobe.Click
        qd02.TriggerStrobe()    ' Trigger flash strobe
        LoseFocus()             ' take focus off clicked button
    End Sub

    '************************************************************************************************
    ' UTILITY FUNCTIONS
    '************************************************************************************************

    Public Sub ErrorDetect(ByVal ErrCode As Integer, ByVal ErrMsg As String) Handles qd02.DeviceError
        MsgBox(ErrMsg, MsgBoxStyle.Critical, "QD02 Error")
        Application.Exit()  ' terminate program
    End Sub

    ' Enable/disable all UI controls, from any thread.
    Private Sub GUIEnable(ByVal enable As Boolean)
        If Me.InvokeRequired Then
            Me.Invoke(New Action(Of Boolean)(AddressOf GUIEnable), enable)
        Else
            Me.Enabled = enable
        End If
    End Sub

    Private Function TextboxToInt(ByVal t As TextBox, ByVal min As Integer, ByVal max As Integer) As Integer
        On Error GoTo valerr
        Dim val As Integer = CInt(t.Text)
        If (val < min) Then Return min
        If (val > max) Then Return max
        Return val
valerr:
        Return min
    End Function

    Private Sub StrobeTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StrobeTimer.Tick
        StrobeTimer.Enabled = False
        btnStrobe.BackColor = SystemColors.Control  ' terminate indication of strobe event
    End Sub

End Class
