﻿Imports System
Imports System.Runtime.InteropServices

'*******************************************************************************************
' File      : qd02_api.vb
' Function  : API for Orled QD02 ring light controllers
' Usage     : Include this file in any VB.NET project that controls Orled ring lights
' Copyright : (C) 2015 Orled, Inc.
'*******************************************************************************************

Module QD02Api

    Private Declare Function QD02_ApiVersionRead Lib "OrledQD02" () As Integer

    Public Const MAX_NUM_DEVICES As Integer = 8     ' Maximum number of ringlights supported by the API

    Public Enum QD02_ERR As Integer    ' Error codes ----------------
        OK = 0                    ' No error
        DEVICE_NOT_FOUND = -1     ' Can't find QD02 device
        DEVICE_OPEN = -2          ' Can't open QD02
        DEVICE_CLOSED = -3        ' QD02 is not open
        READ = -4                 ' Error during device read
        WRITE = -5                ' Error during device write
        INVALID_ARG = -6          ' Illegal logical address or other function argument
        USB_OPEN = -7             ' Problem opening or can't find generic hid api (in Windows, this is hid.dll)
        COMMAND_MISMATCH = -8     ' Command/reply mismatch (results from multiple threads/processes attempting to talk to same device)
        TIMEOUT = -9              ' Timed out waiting for device I/O
        CREATE_RESOURCE = -10     ' Can't create resource (e.g., mutex/semaphore/thread)
        WAS_SHUTDOWN = -11        ' Receive communications have terminated
    End Enum

    Public Function GetApiVersionString() As String
        Dim api As Integer = QD02_ApiVersionRead()
        Return (api >> 24).ToString & "." & ((api >> 16) And &HFF).ToString & "." & (api And &HFF).ToString
    End Function

    Public Function ErrString(ByVal errcode As QD02_ERR) As String    ' Convert error code to string
        Select Case errcode
            Case Is >= 0 : Return ""
            Case QD02_ERR.DEVICE_NOT_FOUND : Return "Device not found"
            Case QD02_ERR.DEVICE_OPEN : Return "Device is already open"
            Case QD02_ERR.DEVICE_CLOSED : Return "Device is not open"
            Case QD02_ERR.READ : Return "Device read failed"
            Case QD02_ERR.WRITE : Return "Device write failed"
            Case QD02_ERR.INVALID_ARG : Return "Invalid function argument"
            Case QD02_ERR.USB_OPEN : Return "Device could not be opened"
            Case QD02_ERR.COMMAND_MISMATCH : Return "Command/reply mismatch"
            Case QD02_ERR.TIMEOUT : Return "Timed out"
            Case QD02_ERR.CREATE_RESOURCE : Return "Can't create resource"
            Case QD02_ERR.WAS_SHUTDOWN : Return "Receiver shutdown"
            Case Else : Return "Unknown error"
        End Select
    End Function

End Module

Class QD02System   ' SYSTEM OF ONE OR MORE RING LIGHT CONTROLLERS ===================================

    Private Declare Function QD02_Enumerate Lib "OrledQD02" () As Integer

    Public Event SystemError(ByVal ErrCode As Integer, ByVal ErrMsg As String)

    Public Function EnumerateRinglights() As Integer
        Dim devmask As Integer = QD02_Enumerate()   ' get device bitmask or errcode
        Select Case devmask
            Case Is > 0 : Return devmask  ' no error: return bitmask
            Case 0 : devmask = QD02_ERR.DEVICE_NOT_FOUND  ' no error and no devices detected, so force error
        End Select
        RaiseEvent SystemError(devmask, ErrString(devmask))  ' notify app of error
        Return 0  ' return empty bitmask
    End Function

End Class

Class QD02Device  ' RING LIGHT CONTROLLER ==============================================

    ' Event flag bitmasks for notification message EventFlags member
    <Flags()> Public Enum EVENT_FLAGS As Byte
        TEMP = 1            ' One or more quadrant temperatures changed. Enable/disable temperature notifications with QD02_MonitorTemp()
        FAULT = 2           ' Fault status changed (may be cable or keypad fault)
        CONSOLE = 4         ' Console state changed, except power (crosshair on/off, intensity change, quadrant enable/disable)
        CONFIG = 8          ' Configuration changed (illumination mode, crosshair time, strobe delay/duration)
        POWER = &H10        ' Power switch state changed
        STROBE = &H20       ' Strobe was activated
        'spare = &H40       '   reserved
        SOFT = &H80         ' Software triggered notification (by QD02_TriggerNotification)
    End Enum

    ' Status flag bitmasks for notification message StatusFlags member
    <Flags()> Public Enum STATUS As Byte
        POWER = &H80        ' Power switch on
        LASERHEAD = &H40    ' Ring light has crosshair lasers
        OVERTEMP = &H10     ' Temperature is too high
        CABLEFAULT = 2      ' Ringlight cable fault
        KEYPADFAULT = 1     ' Keypad fault
    End Enum

    ' Illumination modes
    Public Enum ILLUMINATION_MODE As Byte
        DIGITAL = 0         ' Digital (PWM) intensity control
        ANALOG = 1          ' Analog intensity control
        STROBE = 2          ' Strobe mode
    End Enum

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure NOTIFICATION          ' Notification message returned by QD02_ReadNotification()
        ' Event attributes
        Public EventFlags As EVENT_FLAGS
        Public NotificationID As UInt16
        Public Timestamp As UInt16
        ' Real-time status
        Public QuadrantEnables As Byte
        Public Brightness As Byte
        Public CrosshairEnable As Byte
        Public StatusFlags As STATUS
        ' Quadrant temperatures
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=4)> Public Temperature() As Byte
        ' Configurations
        Public IlluminationMode As ILLUMINATION_MODE
        Public CrosshairOntime As UInt16
        Public StrobeDelay As UInt16
        Public StrobeDuration As UInt16
        Public MaxIntensity As Byte
        ' Statistics
        Public DroppedNotifications As UInt32
    End Structure

    <StructLayout(LayoutKind.Sequential)> _
    Private Structure FIRMWAREINFO          ' Firmware attributes returned by QD02_ReadFirmwareInfo()
        Public RunningBootloader As Byte
        Public FwVersion As UInt32
    End Structure

    ' Storage ---------------------------------

    Private MyLogAddr As Integer                ' Logical device address, set by dip switches on unit
    Private MyHwRev As UShort                   ' Device hardware version
    Private MyGuiContext As Form                ' Application's GUI context

    ' Async notifications monitor thread; receives and processes event messages from the QD02.  
    Private MonitorThread As Threading.Thread = New Threading.Thread(AddressOf MonitorThreadFunction)

    ' API Functions ---------------------------

    Private Declare Function QD02_Open Lib "OrledQD02" (ByVal logaddr As Integer, ByRef HwRev As UInt16) As Integer
    Private Declare Function QD02_Close Lib "OrledQD02" (ByVal logaddr As Integer) As Integer
    Private Declare Function QD02_Shutdown Lib "OrledQD02" (ByVal logaddr As Integer) As Integer
    Private Declare Function QD02_ConfigStrobe Lib "OrledQD02" (ByVal logaddr As Integer, ByVal delay As UInt16, ByVal duration As UInt16) As Integer
    Private Declare Function QD02_ConfigMode Lib "OrledQD02" (ByVal logaddr As Integer, ByVal mode As Byte) As Integer
    Private Declare Function QD02_ConfigMaxIntensity Lib "OrledQD02" (ByVal logaddr As Integer, ByVal intensity As Byte) As Integer
    Private Declare Function QD02_ConfigCrosshair Lib "OrledQD02" (ByVal logaddr As Integer, ByVal ontime As UInt16) As Integer
    Private Declare Function QD02_ConfigSave Lib "OrledQD02" (ByVal logaddr As Integer) As Integer
    Private Declare Function QD02_ConfigRestore Lib "OrledQD02" (ByVal logaddr As Integer) As Integer
    Private Declare Function QD02_ControlPower Lib "OrledQD02" (ByVal logaddr As Integer, ByVal enable As Byte) As Integer
    Private Declare Function QD02_ControlQuadrants Lib "OrledQD02" (ByVal logaddr As Integer, ByVal enables As Byte) As Integer
    Private Declare Function QD02_ControlIntensity Lib "OrledQD02" (ByVal logaddr As Integer, ByVal level As Byte) As Integer
    Private Declare Function QD02_ControlCrosshair Lib "OrledQD02" (ByVal logaddr As Integer, ByVal enable As Byte) As Integer
    Private Declare Function QD02_MonitorTemp Lib "OrledQD02" (ByVal logaddr As Integer, ByVal enable As Byte) As Integer
    Private Declare Function QD02_TriggerStrobe Lib "OrledQD02" (ByVal logaddr As Integer) As Integer
    Private Declare Function QD02_TriggerNotification Lib "OrledQD02" (ByVal logaddr As Integer) As Integer
    Private Declare Function QD02_ReadNotification Lib "OrledQD02" (ByVal logaddr As Integer, ByRef notification As NOTIFICATION, ByVal milliseconds As UInt32) As Integer
    Private Declare Function QD02_ReadFirmwareInfo Lib "OrledQD02" (ByVal logaddr As Integer, ByRef systemstate As FIRMWAREINFO) As Integer

    Public Event DeviceError(ByVal ErrCode As Integer, ByVal ErrMsg As String)
    Public Event ReceivedNotification(ByVal n As NOTIFICATION)

    Public Sub New(ByVal device_logaddr As Integer)  ' Constructor
        MyBase.New()
        MyLogAddr = device_logaddr  ' Save logical device address (matches rotary switch settings on the target QD02)
    End Sub

    Private Sub ErrorDetect(ByVal errcode As Integer)  ' Raise api error to app
        If (errcode < 0) Then RaiseEvent DeviceError(errcode, ErrString(errcode))
    End Sub

    Public Sub Open(ByVal NotificationContext As Form)
        MyGuiContext = NotificationContext                          ' Save GUI context so notifications can be safely passed to it.
        Dim errcode As Integer = QD02_Open(MyLogAddr, MyHwRev)      ' Open the QD02 device and read its hardware version
        MonitorThread.Start()                                       ' Start the notification monitor thread
        ErrorDetect(QD02_TriggerNotification(MyLogAddr))            ' Force QD02 to send notification, to initialize the application GUI (if it cares).
    End Sub

    Public Sub Close()
        QD02_Shutdown(MyLogAddr)    ' Shutdown API comlink. This causes QD02_ReadNotification() to return an error, which will terminate MonitorThread.
        MonitorThread.Join()        ' Wait for MonitorThread to terminate.
        QD02_Close(MyLogAddr)       ' Close the QD02 device.
    End Sub

    ' Notification monitor thread ----------------------------

    Private Delegate Sub GUIInvoker(ByVal n As NOTIFICATION)

    Private Sub MonitorThreadFunction()
        Dim n As New QD02Device.NOTIFICATION  ' create new notification buffer
        Const INFINITE_WAIT = &HFFFFFFFFL                                               ' Repeat until error or device closed:
        While (QD02_ReadNotification(MyLogAddr, n, INFINITE_WAIT) = QD02_ERR.OK)        '   Wait for notification message from device.
            MyGuiContext.BeginInvoke(New GUIInvoker(AddressOf RaiseNotification), n)    '   Schedule RaiseEvent in GUI's context.
        End While
    End Sub

    Private Sub RaiseNotification(ByVal n As NOTIFICATION)    ' Forward received QD02 event notification to application
        RaiseEvent ReceivedNotification(n)
    End Sub

    ' Public methods and properties ---------------------------

    Public Sub ConfigSave()  ' Save current config as defaults
        ErrorDetect(QD02_ConfigSave(MyLogAddr))
    End Sub

    Public Sub ConfigRestore()  ' Restore config to defaults
        ErrorDetect(QD02_ConfigRestore(MyLogAddr))
    End Sub

    Public WriteOnly Property Power() As Boolean    ' Control device power
        Set(ByVal value As Boolean)
            ErrorDetect(QD02_ControlPower(MyLogAddr, value))
        End Set
    End Property

    Public WriteOnly Property Intensity() As Byte   ' Control light intensity
        Set(ByVal Level As Byte)
            ErrorDetect(QD02_ControlIntensity(MyLogAddr, Level))
        End Set
    End Property

    Public WriteOnly Property Quadrants() As Byte   ' Control quadrant enables
        Set(ByVal EnableFlags As Byte)
            ErrorDetect(QD02_ControlQuadrants(MyLogAddr, EnableFlags))
        End Set
    End Property

    Public WriteOnly Property IlluminationMode() As ILLUMINATION_MODE   ' Set illumination mode
        Set(ByVal value As ILLUMINATION_MODE)
            ErrorDetect(QD02_ConfigMode(MyLogAddr, value))
        End Set
    End Property

    Public WriteOnly Property Crosshair() As Boolean    ' Turn laser crosshairs on/off
        Set(ByVal enable As Boolean)
            ErrorDetect(QD02_ControlCrosshair(MyLogAddr, enable))
        End Set
    End Property

    Public WriteOnly Property CrosshairOntime() As UShort   ' Configure crosshair on-time
        Set(ByVal value As UShort)
            ErrorDetect(QD02_ConfigCrosshair(MyLogAddr, value))
        End Set
    End Property

    Public Sub ConfigStrobe(ByVal delay As UShort, ByVal duration As UShort)    ' Configure flash strobe (duration = 4us/bit)
        ErrorDetect(QD02_ConfigStrobe(MyLogAddr, delay, duration))
    End Sub

    Public WriteOnly Property MaxIntensity() As Byte    ' Configure maximum light intensity
        Set(ByVal value As Byte)
            ErrorDetect(QD02_ConfigMaxIntensity(MyLogAddr, value))
        End Set
    End Property

    Public Sub TriggerStrobe()  ' Initiate a flash strobe
        ErrorDetect(QD02_TriggerStrobe(MyLogAddr))
    End Sub

    Public Sub MonitorTemp(ByVal enable As Byte)    ' Turn on/off temperature change notifications
        ErrorDetect(QD02_MonitorTemp(MyLogAddr, enable))
    End Sub

    Public ReadOnly Property FirmwareVersion() As String    ' Get device firmware version
        Get
            Dim fw As FIRMWAREINFO
            ErrorDetect(QD02_ReadFirmwareInfo(MyLogAddr, fw))
            Return (fw.FwVersion >> 24).ToString & "." & ((fw.FwVersion >> 16) And &HFF).ToString & "." & (fw.FwVersion And &HFF).ToString
        End Get
    End Property

    Public ReadOnly Property HardwareRev() As String    ' Get device hardware version
        Get
            Dim RevLetter As String = "0ABCDEFGHJKLMNPRT"
            If (MyHwRev < Len(RevLetter)) Then Return Mid$(RevLetter, MyHwRev + 1, 1) Else Return "?"
        End Get
    End Property

End Class
